package cn.newfeifan.mall.module.sale.service.merchantapply;

import cn.newfeifan.mall.framework.common.exception.ErrorCode;
import cn.newfeifan.mall.module.member.api.user.MemberUserApi;
import cn.newfeifan.mall.module.member.api.user.dto.MemberUserRespDTO;
import cn.newfeifan.mall.module.member.enums.DictTypeConstants;
import cn.newfeifan.mall.module.sale.controller.app.merchantapply.vo.MerchantApplyPageReqVO;
import cn.newfeifan.mall.module.sale.controller.app.merchantapply.vo.MerchantApplySaveReqVO;
import cn.newfeifan.mall.module.sale.enums.MerchantApplyCheckStatusEnum;
import cn.newfeifan.mall.module.system.service.user.AdminUserService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;

import cn.newfeifan.mall.module.sale.dal.dataobject.merchantapply.MerchantApplyDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.sale.dal.mysql.merchantapply.MerchantApplyMapper;


import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.newfeifan.mall.module.sale.enums.ErrorCodeConstants.*;

/**
 * 商户申请 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class MerchantApplyServiceImpl implements MerchantApplyService {

    @Resource
    private MerchantApplyMapper merchantApplyMapper;

    @Resource
    private AdminUserService usersService;

    @Resource
    private MemberUserApi memberUserApi;

    @Override
    public Long createMerchantApply(MerchantApplySaveReqVO createReqVO) {

        // 校验存在
        if (usersService.getUserByMobile(createReqVO.getContactNumber()) != null) {
            ErrorCode ERROR = new ErrorCode(1_002_029_003, "联络人账户已存在");
            throw exception(ERROR);
        }

        // 插入
        MerchantApplyDO merchantApply = BeanUtils.toBean(createReqVO, MerchantApplyDO.class);
        merchantApply.setApplyMemberUserId(getLoginUserId());

        // 默认log
        if (createReqVO.getLogoUrl() == null || createReqVO.getLogoUrl().isEmpty()) {
            merchantApply.setLogoUrl(DictTypeConstants.DEFAULT_AVATAR);
        }

        merchantApplyMapper.insert(merchantApply);
        // 返回
        return merchantApply.getId();
    }

    @Override
    public void updateMerchantApply(MerchantApplySaveReqVO updateReqVO) {

        MerchantApplyDO merchantApplyDO = merchantApplyMapper.selectById(updateReqVO.getId());

        // 联系人手机号校验
        if (!merchantApplyDO.getContactNumber().equals(updateReqVO.getContactNumber()) && usersService.getUserByMobile(updateReqVO.getContactNumber()) != null) {
            ErrorCode ERROR = new ErrorCode(1_002_029_003, "联络人账户已存在");
            throw exception(ERROR);
        }

        updateReqVO.setApplyMemberUserId(getLoginUserId());
        // 校验存在
        validateMerchantApplyExists(updateReqVO.getId());
        // 更新
        MerchantApplyDO updateObj = BeanUtils.toBean(updateReqVO, MerchantApplyDO.class);

        // 默认log
        if (updateObj.getLogoUrl() == null || updateObj.getLogoUrl().isEmpty()) {
            updateObj.setLogoUrl(DictTypeConstants.DEFAULT_AVATAR);
        }

        // 所有变动都需要重新审核
        updateObj.setCheckStatus(MerchantApplyCheckStatusEnum.WAIT_AUDIT.getType());

        merchantApplyMapper.updateById(updateObj);
    }

    @Override
    public void deleteMerchantApply(Long id) {
        // 校验存在
        validateMerchantApplyExists(id);
        // 删除
        merchantApplyMapper.deleteById(id);
    }

    private void validateMerchantApplyExists(Long id) {
        if (merchantApplyMapper.selectById(id) == null) {
            throw exception(MERCHANT_APPLY_NOT_EXISTS);
        }
    }

    @Override
    public MerchantApplyDO getMerchantApply(Long id) {
        return merchantApplyMapper.selectById(id);
    }

    @Override
    public PageResult<MerchantApplyDO> getMerchantApplyPage(MerchantApplyPageReqVO pageReqVO) {
        return merchantApplyMapper.selectPage(pageReqVO);
    }

    @Override
    public MerchantApplyDO getMerchantApplyBYUserId(Long loginUserId) {
        return merchantApplyMapper.selectOne(MerchantApplyDO::getApplyMemberUserId, loginUserId);
    }

    @Override
    public Boolean isSystemUserByMobile() {
        // 获取消费者手机号
        MemberUserRespDTO user = memberUserApi.getUser(getLoginUserId());
        // 获取申请入驻信息
        MerchantApplyDO merchantApplyDO = merchantApplyMapper.selectOne(MerchantApplyDO::getApplyMemberUserId, getLoginUserId());

        // 判断是否有系统用户
        return usersService.getUserByMobile(user.getMobile()) != null && merchantApplyDO == null;
    }

}